#include <HashMapSimple.mqh>

#define NO_VALUE -1

class Indicators
{
  protected:
    static HashMapSimple<string,int> mapOfInstances;
    
  public:
    static double jMA(string symbol, const ENUM_TIMEFRAMES period, const int ma_period, const int ma_shift, const ENUM_MA_METHOD ma_method, const ENUM_APPLIED_PRICE applied_price, const int bar)
    {
      if(symbol == NULL) symbol = _Symbol;
      string key = StringFormat("iMA-%s-%d-%d-%d-%d-%d", symbol, period, ma_period, ma_shift, ma_method, applied_price);
      int handle = mapOfInstances[key];
      if(handle == NO_VALUE)
      {
        handle = iMA(symbol, period, ma_period, ma_shift, ma_method, applied_price);
        if(handle != INVALID_HANDLE)
        {
          mapOfInstances.add(key, handle);
        }
        else
        {
          // Print("ERROR: Can't create iMA: ", GetLastError());
        }
      }
      else
      {
        double result[1];
        if(CopyBuffer(handle, 0, bar, 1, result) == 1)
        {
          return result[0];
        }
      }
      return EMPTY_VALUE;
    }
    
    static double jRSI(string symbol, const ENUM_TIMEFRAMES timeframe, const int period, const ENUM_APPLIED_PRICE applied_price, const int bar)
    {
      if(symbol == NULL) symbol = _Symbol;
      string key = StringFormat("iRSI-%s-%d-%d-%d", symbol, timeframe, period, applied_price);
      int handle = mapOfInstances[key];
      if(handle == NO_VALUE)
      {
        handle = iRSI(symbol, timeframe, period, applied_price);
        if(handle != INVALID_HANDLE)
        {
          mapOfInstances.add(key, handle);
        }
      }
      else
      {
        double result[1];
        if(CopyBuffer(handle, 0, bar, 1, result) == 1)
        {
          return result[0];
        }
      }
      return EMPTY_VALUE;
    }
    
    template<typename T1, typename T2>
    static double jCustom(string symbol, const ENUM_TIMEFRAMES period, const string name, T1 t1, T2 t2, const int buffer, const int bar)
    {
      if(symbol == NULL) symbol = _Symbol;
      string key = StringFormat("iCustom-%s-%d-%s-%s-%s", symbol, period, name, (string)t1, (string)t2);
      int index = mapOfInstances[key];
      if(index == NO_VALUE)
      {
        int handle = iCustom(symbol, period, name, t1, t2);
        if(handle != INVALID_HANDLE)
        {
          mapOfInstances.add(key, handle);
        }
      }
      else
      {
        double result[1];
        if(CopyBuffer(index, buffer, bar, 1, result) == 1)
        {
          return result[0];
        }
      }
      return EMPTY_VALUE;
    }
    
    template<typename T1, typename T2, typename T3, typename T4, typename T5>
    static double jCustom(string symbol, const ENUM_TIMEFRAMES period, const string name, T1 t1, T2 t2, T3 t3, T4 t4, T5 t5, const int buffer, const int bar)
    {
      if(symbol == NULL) symbol = _Symbol;
      string key = StringFormat("iCustom-%s-%d-%s-%s-%s-%s-%s-%s", symbol, period, name, (string)t1, (string)t2, (string)t3, (string)t4, (string)t5);
      int index = mapOfInstances[key];
      if(index == NO_VALUE)
      {
        int handle = iCustom(symbol, period, name, t1, t2, t3, t4, t5);
        if(handle != INVALID_HANDLE)
        {
          mapOfInstances.add(key, handle);
        }
      }
      else
      {
        double result[1];
        if(CopyBuffer(index, buffer, bar, 1, result) == 1)
        {
          return result[0];
        }
      }
      return EMPTY_VALUE;
    }
    
};

HashMapSimple<string,int> Indicators::mapOfInstances("", NO_VALUE);

#define iMA Indicators::jMA
#define iRSI Indicators::jRSI
#define iCustom Indicators::jCustom
